#include <exec/exec.h>
#include <dos/dosextens.h>
#include <dos/rdargs.h>
#include <dos/datetime.h>
#include <clib/exec_protos.h>
#include <clib/dos_protos.h>
#include <clib/alib_protos.h>
#include <time.h>
#include <string.h>

typedef struct
{
    char path[256];
    time_t stamp;
}
FAXITEM;

typedef struct
{
    struct Node mn;
    FAXITEM f;
}
FLIST;

void FreeList (struct List *ml)
{
    FLIST *wn, *nn;

    wn = (FLIST *) ml->lh_Head;
    while (nn = (FLIST *) wn->mn.ln_Succ)
    {
	FreeMem (wn->mn.ln_Name, 80);
	FreeMem (wn, sizeof (FLIST));
	wn = nn;
    }
    FreeMem (ml, sizeof (struct List));

}

struct List *BuildFaxList (char *file)
{
    FLIST *pn;
    struct List *ml;
    ULONG fh;
    FAXITEM f;
    short nl = 0;

    if (ml = AllocMem (sizeof (struct List), MEMF_ANY | MEMF_CLEAR))
    {
	NewList ((struct List *) ml);
	if (fh = Open ("qfax:QFAXThing", MODE_OLDFILE))
	{
	    while (FRead (fh, &f, sizeof (FAXITEM), 1) > 0)
	    {
		FLIST *nn;
		char *p;
		short ll;

		nn = AllocMem (sizeof (FLIST), MEMF_ANY | MEMF_CLEAR);
		memcpy (&(nn->f), &f, sizeof (FAXITEM));
		p = FilePart (nn->f.path);
		for (ll = 0; p > nn->f.path && ll < 2;)
		{
		    --p;
		    if (*p == '/' || *p == ':')
		    {
			ll++;
		    }
		}

		if (ll == 2)
		{
		    p++;
		}
		if ((ll = strlen (p)) > nl)
		{
		    nl = ll;
		}
		nn->mn.ln_Name = AllocMem (80, MEMF_ANY | MEMF_CLEAR);
		strcpy (nn->mn.ln_Name, p);
		AddTail (ml, (struct Node *) nn);
	    }

	    for (pn = (FLIST *) ml->lh_Head; pn->mn.ln_Succ; pn = (FLIST *) pn->mn.ln_Succ)
	    {
		extern long _TimeCompensation;
		char stim[32];
		struct DateTime dt;
		long t;
		char *p;

		p = strchr (pn->mn.ln_Name, '\0');
		while (p && p <= (pn->mn.ln_Name + nl))
		{
		    *p++ = ' ';
		}

		t = pn->f.stamp - _TimeCompensation;
		dt.dat_Stamp.ds_Days = (unsigned long) t / 86400;
		dt.dat_Stamp.ds_Minute = ((unsigned long) t / 60) % 1440;
		dt.dat_Stamp.ds_Tick = ((unsigned long) t % 60) * 50;
		dt.dat_Format = FORMAT_DOS;
		dt.dat_Flags = 0;
		dt.dat_StrDay = NULL;
		dt.dat_StrDate = stim;
		dt.dat_StrTime = stim + 16;
		DateToStr (&dt);
		strncpy (p, stim, 9);
		*(p + 9) = ' ';
		strncpy (p + 10, stim + 16, 8);
		*(p + 18) = '\0';
	    }
	    Close (fh);
	}
    }
    return ml;
}

#ifdef TEST
__stkargs void _exit (int);
__stkargs int _main (void)
{
    FLIST *pn;
    struct List *ml;

    if (ml = BuildFaxList ("qfax:QFAXThing"))
    {
	for (pn = (FLIST *) ml->lh_Head; pn->mn.ln_Succ; pn = (FLIST *) pn->mn.ln_Succ)
	{
	    Printf ("%s\n", pn->mn.ln_Name);
	}
	FreeList (ml);
    }
    _exit (0);
}
#endif
