
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>

#include <fcntl.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <termios.h>

#define BITSON ( ~0 )

#include "qheader.h"
#include "qfaxparse.h"
#include "qfax.h"

FAXDATA qd;

#define TMP "/tmp/"
#define DOT '.'

static unsigned char rtabl[] = {
0x00,0x80,0x40,0xc0,0x20,0xa0,0x60,0xe0,
0x10,0x90,0x50,0xd0,0x30,0xb0,0x70,0xf0,
0x08,0x88,0x48,0xc8,0x28,0xa8,0x68,0xe8,
0x18,0x98,0x58,0xd8,0x38,0xb8,0x78,0xf8,
0x04,0x84,0x44,0xc4,0x24,0xa4,0x64,0xe4,
0x14,0x94,0x54,0xd4,0x34,0xb4,0x74,0xf4,
0x0c,0x8c,0x4c,0xcc,0x2c,0xac,0x6c,0xec,
0x1c,0x9c,0x5c,0xdc,0x3c,0xbc,0x7c,0xfc,
0x02,0x82,0x42,0xc2,0x22,0xa2,0x62,0xe2,
0x12,0x92,0x52,0xd2,0x32,0xb2,0x72,0xf2,
0x0a,0x8a,0x4a,0xca,0x2a,0xaa,0x6a,0xea,
0x1a,0x9a,0x5a,0xda,0x3a,0xba,0x7a,0xfa,
0x06,0x86,0x46,0xc6,0x26,0xa6,0x66,0xe6,
0x16,0x96,0x56,0xd6,0x36,0xb6,0x76,0xf6,
0x0e,0x8e,0x4e,0xce,0x2e,0xae,0x6e,0xee,
0x1e,0x9e,0x5e,0xde,0x3e,0xbe,0x7e,0xfe,
0x01,0x81,0x41,0xc1,0x21,0xa1,0x61,0xe1,
0x11,0x91,0x51,0xd1,0x31,0xb1,0x71,0xf1,
0x09,0x89,0x49,0xc9,0x29,0xa9,0x69,0xe9,
0x19,0x99,0x59,0xd9,0x39,0xb9,0x79,0xf9,
0x05,0x85,0x45,0xc5,0x25,0xa5,0x65,0xe5,
0x15,0x95,0x55,0xd5,0x35,0xb5,0x75,0xf5,
0x0d,0x8d,0x4d,0xcd,0x2d,0xad,0x6d,0xed,
0x1d,0x9d,0x5d,0xdd,0x3d,0xbd,0x7d,0xfd,
0x03,0x83,0x43,0xc3,0x23,0xa3,0x63,0xe3,
0x13,0x93,0x53,0xd3,0x33,0xb3,0x73,0xf3,
0x0b,0x8b,0x4b,0xcb,0x2b,0xab,0x6b,0xeb,
0x1b,0x9b,0x5b,0xdb,0x3b,0xbb,0x7b,0xfb,
0x07,0x87,0x47,0xc7,0x27,0xa7,0x67,0xe7,
0x17,0x97,0x57,0xd7,0x37,0xb7,0x77,0xf7,
0x0f,0x8f,0x4f,0xcf,0x2f,0xaf,0x6f,0xef,
0x1f,0x9f,0x5f,0xdf,0x3f,0xbf,0x7f,0xff};



short msg (short err, char *s,...)
{
    va_list ap;
    time_t et;
    char *t, txt[132];
    struct tm *p;

    et = time (NULL);
    p = localtime (&et);

    va_start (ap, s);

    sprintf (txt, "%02d:%02d:%02d ", p->tm_hour, p->tm_min, p->tm_sec);
    t = txt + strlen (txt);
    vsprintf (t, s, ap);
    va_end (ap);

    puts (txt);
    return err;
}

static void rl_ttyset (int flag)
{
    static struct termios old;
    struct termios new;

    if (flag)
    {
	(void) tcgetattr (0, &old);
	new = old;
	new.c_cc[VINTR] = BITSON;
	new.c_cc[VQUIT] = BITSON;
	new.c_lflag &= ~(ECHO | ICANON);
	new.c_iflag &= ~(ISTRIP | INPCK);
	new.c_cc[VMIN] = 0;
	new.c_cc[VTIME] = 0;
	(void) tcsetattr (0, TCSANOW, &new);
    }
    else
	(void) tcsetattr (0, TCSANOW, &old);
}

int Wgetch(void)
{
    char c = 0;
    rl_ttyset (1);
    if(read (0, &c, 1) != 1) c = 0;
    rl_ttyset (0);
    return (int)c;
}


int readser(char * buf)
{
    char *p = buf;
    unsigned char c;    
    char x;
    int dog = 0;
    short nw = 0;

    *p = 0;
    while(1)
    {
        if(SRead((char *) &c, 1, 1) == 1)
        {
	    if(c > 31 && c < 128)
	    {
		fputc((int)c, stdout);
	    }
	    else if(c != 13)
	    {
		printf("<%02x>", c);
	    }
	    fflush(stdout);
	    
            if(c < 32)
            {
                if(c == 13)
                {
		    fputc(10, stdout);
                    *p = 0;
                    break;
                }
            }
            else
            { 
                *p++ = c;
            }
	}
	else
	{
	    nw++;
	    if(nw == 200 && p == buf)
	    {
	        return 0;
	    }
	}
        if((x = Wgetch()))
        {
            if(x == 3)
            {
                exit (0);
            }
            if (x == 'R' || x == 'r')
            {
                SWrite("\r\nRING\r\n", -1);
                msg(0, "Ringing ....");
		break;
	    }
	    else if(x == 'G' || x == 'g')
	    {
	        dog = 1;
	        break;
	    }
        }
    }
    return dog;
}

int DummyRecv(FILE *f)
{
    unsigned char c;
    short nulls = 0;
    short dle = 0;
    short done;
    int nc = 0;
    char txt[80];
    unsigned char fc = 0;
    
    sprintf (txt, "\r\n+FDCS:%d,%d,0,2,%d,0,0,0\r\n", qd.res, qd.vmodem, qd.code);
    SWrite( txt, -1);
    SWrite("\r\nCONNECT\r\n", -1);

    if (qd.startchar)
    {
	msg (0, "Send start char <%x>", qd.startchar);
	SWrite (&qd.startchar, 1);
    }

    for (done = 0; done == 0;)
    {
	if (SRead ((char *) &c, 1, 150) == 1)
	{
	    nc++;
            if(c)
            {
                if(fc == 0)
                {
                    fc = c;
                }
                nulls = 0;
            }
            else if(++nulls > 5)
            {
                continue;
            }
            
	    if (dle)
	    {
		if (c == 0x10)
		{
		    fputc(c, f);
		}
		else
		{
		    if (c == 0x3)
		    {
			done = 1;
		    }
		    else
		    {
			msg (0, "[%02x] received after DLE", c);
		    }
		}
		dle = 0;
	    }
	    else
	    {
		if (c == 0x10)
		{
		    dle = 1;
		}
		else
		{
		    fputc(c, f);
		}
	    }
	}
	else
	{
	    done = 1;
	    msg (0, "fax device read timeout %d", nc);
	}
    }
    SWrite("\r\nOK\r\n", -1);
    return fc;
}
void WriteHeader(FILE *f, time_t t, int wd, int vr, int df, int ln, int pg, short fc)
{
    QFAXHEADER h;

    memset(&h, '\0', sizeof(h));
    h.title = QFAXTITLE;
    h.version = QFAXVERS;
    h.page = ln;
    h.pagemax = pg;
    h.width = wd;
    h.resolution = vr;
    h.coding = df;
    strcpy (h.tsi, "test fax");
    h.stamp = t;
    h.lsb = (fc & 1) ? 0 : 1;
    HeaderEndianise(&h);
    fwrite (&h, 1, QFAXHEADLEN, f);
}

void DummySend(FILE *fp, short fbor, short hbor)
{
    int c;
    unsigned char p;
    int err = 0;
    short errc = 0;
    
    SWrite("\r\nCONNECT\r\n", -1);
    InitRTC(hbor, 6);

    if (qd.wait2send)
    {
       while (!err)
       {
	   msg (0, "Waiting for start");
	   if (SRead ((char *)&p, 1, 200) == 0)
	   {
	       err = msg (3, "timed out waiting to send data");
	   }
	   else if (p == 0x11 || p == 0x12)
	   {
	       msg (0, "Starting after [%02x]", p);
	       break;
	   }
	   else
	   {
	       msg (0, "[%02x] received while waiting to send data", p);
	       errc++;
	       if(errc > 5)
	       {
		   err = 2;
	       }
	   }
       }
   }

    if(!err)
    {
#if 0
	while((c = fgetc(fp)) != EOF)
	{
	    p = (fbor == hbor) ? c : rtabl[c];
	    
	    if(p == 0x10)
	    {
		SWrite((char *)&p, 1);
	    }
	    SWrite((char *)&p, 1);
	    if(CheckRTC(c))
	    {
		break;
	    }
	}
	SWrite("\x10\003", 2);
#else
#define SNDBUFSIZE 1024
	unsigned char *op;
	unsigned char obuf[SNDBUFSIZE + 5];
	short done;

	op = obuf;
	
        for(done =0; !done;)
	{
	    if((c = fgetc (fp)) == EOF)
		done = 1;
	    if (!done)
	    {
		if (CheckRTC (c))
		{
		    int nn = GetFaxRTC(NULL);
		    msg(0, "RTC at %d", nn);
		    done = 1;
		}
                if (fbor != hbor)  c = rtabl[c];
		*op++ = c;
		if (c == 0x10)
		    *op++ = 0x10;
	    }

	    if (op >= (obuf + SNDBUFSIZE) || done)
	    {
		if (done)
		{
		    *op++ = 0x10;
		    *op++ = 3;
		}
		SWrite ((char *) obuf, (op - obuf));
		op = obuf;
	    }
	}
#endif
    }

}

int mycomp (char **s1, char **s2)
{
    return (strcmp (*s1, *s2));
}


int main (int ac, char **av)
{
    int c;
    char txt[80];
    short cont = 0;
    short done = 0;
    char *pf;
    int dog;
    int br = 0;
    int pg = 0;
    int wd=0, vr=0, df=0, ln = 0;
    int rgs = 0, rrgs;
    int iw = 0;
    unsigned char nt = 0;
    FILE *fp = NULL;
    time_t t = 0;
    short lp = 0;
    short fc = 0;
    short fbor = 0;
  
    QFAXHEADER h;
    char *ps;
    char *sdev = NULL;
    
    while ((c = getopt (ac, av, "b:cCr:R:s:xd:")) != -1)
    {
        switch(c)
        {
	    case 'd':
		sdev = optarg;
		break;
            case 'r':
                lp = iw = 1;
            case 'R':
                rgs = atoi(optarg);
            case 'c':
            case 'C':
                cont = 1;
                break;
            case 'b':
                br = atoi(optarg);
                break;
        }
    }

    pf = *(av+optind);
    
    FaxParams (&qd);
    if(br) qd.baud = br;
    if(sdev) strcpy(qd.serdev, sdev);

    tty_open (qd.serdev, qd.baud, 1);

    rrgs = rgs;

    while(SRead(txt, 1, 10) == 1)
        ; /* suck */
        
    printf("c2test %s %d\n", qd.serdev, qd.baud);
    
    while (!done)
    {
        dog = readser(txt);
        if(*txt) lp = 1;
        
        if(strcmp(txt, "ATA") == 0 || dog)
        {
            if(dog == 0)
            {
                SWrite("\r\nCONNECT FAX\r\n", -1);
                SWrite("\r\n+FCON\r\n", -1);
            }
            else
            {
                msg(0, "Option G");
            }

            sprintf(txt, "\r\n+FTSI: %s\r\n", qd.LocalId);
            SWrite( txt, -1);
            sprintf (txt, "\r\n+FDCS:%d,%d,0,2,%d,0,0,0\r\n", qd.res, qd.vmodem, qd.code);
            SWrite( txt, -1);
            SWrite("\r\nOK\r\n", -1);
            continue;
        }

        if(strncmp(txt,"AT+FDCC", 7) == 0)
        {
            /*             vr br  wd ln, df */
            sscanf(txt+8, "%d,%*d,%d,%d,%d", &vr, &wd, &ln, &df);
        }

        if((ps = strstr(txt,"AT+FBOR=")))
        {
            fbor = *(ps+8) - '0';
        }

        if(strcmp(txt,"AT+FDR") == 0)
        {
            if(fp == NULL)
            {
                char txt[80];
                fp = fopen(pf, "r");
                fread (&h, 1, QFAXHEADLEN, fp);
		HeaderEndianise(&h);
                sprintf (txt, "\r\n+FDCS:%d,5,%d,%d,%d,0,0,0\r\n",
                            h.resolution, h.width, h.page, h.coding);
                pg = h.pagemax;
                SWrite( txt, -1);
                if(h.version < QFAXFLSB)
                {
                    h.lsb = 0;
                }
                else
                {
                    h.lsb &= 1;
                }
            }
            if(fp)
            {
                if(pg)
                {
                    DummySend(fp, fbor, h.lsb);
                    SWrite("\r\n+FPTS:1\r\n", -1);
                    sprintf(txt,"\r\n+FET:%c\r\n", (pg == 1) ? '0' : '2');
                    SWrite(txt, -1);
                    SWrite("\r\nOK\r\n", -1);
                }
                else
                {
                    SWrite("\r\n+FHNG:00\r\n", -1);
                    SWrite("\r\nOK\r\n", -1);
                    fclose(fp);
                    fp = NULL;       
                }
                pg--;
	    }
            continue;
        }
        if(strncmp(txt,"ATD", 3) == 0)
        {
            pg = 0;
            nt++;
            SWrite("\r\nAT+FCON\r\n", -1);
            sprintf(txt, "\r\n+FCSI: %s\r\n", qd.LocalId);
            SWrite( txt, -1);
            sprintf (txt, "\r\n+FDIS:%d,%d,0,2,%d,0,0,0\r\n", qd.res, qd.vmodem, qd.code);
            SWrite( txt, -1);
            SWrite("\r\nOK\r\n", -1);
            continue;
        }

        if(strcmp(txt,"AT+FDT") == 0)
        {
            char temp[40];
            
            if(fp == NULL)
            {
                pg = 0;
                t = time(NULL);
                sprintf(temp, "%sdemo_%03d%cfax", TMP, nt, DOT);            
                fp = fopen(temp, "w");
            }
            fc = DummyRecv(fp);
            continue;
	}            

        if(strncmp(txt,"AT+FET",6) == 0)
        {
            pg++;
            SWrite("\r\nFPTS:1\r\n", -1);
            if(*(txt+7) == '2')
            {
                SWrite("\r\nFHNG:0\r\n", -1);
                fseek(fp, 0, 0);
                WriteHeader (fp, t, wd, vr, df, ln, pg, fc);
                fclose(fp);
                fp = NULL;            
	    }
            SWrite("\r\nOK\r\n", -1);
            continue;          
        } 

        if(strcmp(txt,"ATH") == 0)
        {
            SWrite("\r\nOK\r\n", -1);
            if(cont == 0)
            {
		sleep(1);
                done = 1;
            }
        }
        if(strncmp(txt, "AT", 2) == 0)
        {
            SWrite("\r\nOK\r\n", -1);
        }
        if(lp && *txt == 0 && rrgs)
        {
                 iw++;
                if(iw == 2)
                {
                    while(rgs--)
                    {
                        SWrite("\r\nRING\r\n", -1);
                    }
                }
                else if (iw == 3)
                {
                    exit(0);
                }
        }
    }
    return 0;
}

