#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>

#define LF_PATH "/var/lock"
#define LF_PREFIX "LCK.."

char * lock_dev (char *dev, pid_t *lockpid)
{
    int lfd;
    pid_t pid, lckpid;
    char *modemname;
    char lnam [FILENAME_MAX];
    char ltmp [FILENAME_MAX];

    *lockpid = 0;
    pid = getpid();
    
    modemname = strrchr(dev, '/');
    sprintf(lnam, "%s/%s%s", LF_PATH, LF_PREFIX, 
	    (modemname ? (modemname + 1) : dev));

    sprintf(ltmp, "%s/%s%d", LF_PATH, "LTMP.", pid);
    unlink(ltmp);
    if ((lfd = creat(ltmp, 0644)) == -1) 
    {
	return NULL;
    }

    write(lfd, (char*)&pid, sizeof(pid));
    close(lfd);

  relink:
    if (link(ltmp, lnam) == 0)
    {
	unlink(ltmp);
	*lockpid = pid;
	return strdup(lnam);
    }

    if ((lfd = open(lnam, O_RDONLY)) != -1) 
    {
	if (read(lfd, (char *)&lckpid, sizeof(lckpid)) == sizeof(lckpid))
	{
	    *lockpid = (pid_t) lckpid;
	    if (kill(lckpid, 0) == 0)
	    {
		unlink(ltmp);
		return NULL;
	    }
	}
    }

    if (unlink(lnam)) 
    {
	unlink(ltmp);
	return NULL;
    }
    goto relink;
}

void unlock_dev(char *lnam, pid_t lockpid)
{
  /* Don't remove the lock file unless it's the one we created */
  if (getpid() == lockpid)
  {
      unlink(lnam);
      free(lnam);
  }
}
#ifdef TEST
int main(int ac, char **av)
{
    char *nam;
    pid_t pid;
    
    if(ac == 2)
    {
	nam = lock_dev(*(av+1), &pid);
	if(nam)
	{
	    printf("Created lock %s for %d %x\n", nam, pid, pid);
	}
	else if(pid)
	{
	    printf("Locked by %x %d\n", pid, pid);
	}
	sleep(60);
    }
    
}
#endif
