/*
 * $Id: amigaserial.c,v 1.1 1994/12/16 15:07:54 jrh Exp $
 * $Log: amigaserial.c,v $
 * Revision 1.1  1994/12/16  15:07:54  jrh
 * Initial revision
 *
 * Revision 1.1  1994/05/28  21:51:27  jrh
 * Initial Revision
 *
 */

#include <dos/dos.h>
#include <exec/types.h>
#include <exec/io.h>
#include <exec/devices.h>
#include <devices/serial.h>
#include <devices/timer.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <clib/dos_protos.h>
#include <clib/exec_protos.h>
#include <clib/alib_protos.h>

#ifdef QFAX
#include "qfax.h"
#else
#define Prototype extern
#endif

static struct IORequest *DeviceOpen(char *, int, struct IORequest *, int);
static struct IORequest *OpenIORequest (char *, int);
static void CloseSerial (void);
static struct IORequest *Open_Device(char *, int, int, int);
static void Close_Device (struct IORequest *);

static struct IOExtSer *SerReq = NULL;
static struct timerequest *TimReq = NULL;

static struct IOExtSer saveps;

#include "qfaxparse.h"
#ifdef DEBUG
static struct {
        char * serdev;
        int devunit;
} qd = {"duart.device", 1};
#else
 extern FAXDATA qd;
#endif

void SerialExit(void)
{
    if(SerReq)
    {
        char c;

        while(SRead(&c, 1, 0) == 1)
            ;
        SerReq->io_Baud = saveps.io_Baud;
        SerReq->io_SerFlags = saveps.io_SerFlags;
        SerReq->io_ReadLen = saveps.io_ReadLen;
        SerReq->io_WriteLen = saveps.io_WriteLen;
        SerReq->IOSer.io_Command = SDCMD_SETPARAMS;
        DoIO((struct IORequest *) SerReq);
        CloseSerial();
   }
    if (TimReq)
    {
        Close_Device ((struct IORequest *) TimReq);
        TimReq = NULL;
    }
}
struct IOExtSer *OpenSerial (int flag)
{
    SerReq = (struct IOExtSer *)OpenIORequest(qd.serdev, sizeof (struct IOExtSer));
    if(SerReq)
    {
        SerReq->io_SerFlags = (SERF_7WIRE | SERF_SHARED);
        SerReq = (struct IOExtSer *) DeviceOpen (qd.serdev, qd.devunit, (struct IORequest *)SerReq, 0);
        SerReq->IOSer.io_Command = SDCMD_QUERY;
        DoIO ((struct IORequest *) SerReq);
        saveps = *SerReq;

        TimReq = (struct TimReq *) Open_Device (TIMERNAME, sizeof (struct timerequest), UNIT_VBLANK, 0);
    }
    return SerReq;
}

static void CloseSerial (void)
{
    Close_Device ((struct IORequest *) SerReq);
    SerReq = NULL;
}

/*
 * Timeout +ve, then wait for timeout
 * Timeout 0    then just return actual
 * Timeout -ve  wait forever
 */

int SRead (char *buf, int nchar, int timeout)
{
    int TimeSig = 0, SerSig = 0;
    int SigMask;
    int rval = 0;

    if(!SerReq)
    {
        return -1;
    }

    SerReq->IOSer.io_Command = SDCMD_QUERY;
    DoIO ((struct IORequest *) SerReq);

    if(SerReq->IOSer.io_Actual >= nchar)
    {
        SerReq->IOSer.io_Length = nchar;
        SerReq->IOSer.io_Data = buf;
        SerReq->IOSer.io_Command = CMD_READ;
        DoIO ((struct IORequest *) SerReq);
    }
    else if(timeout > 0)
    {
	SerReq->IOSer.io_Length = nchar;
	SerReq->IOSer.io_Data = buf;
	SerReq->IOSer.io_Command = CMD_READ;
	SerSig = 1 << (SerReq->IOSer.io_Message.mn_ReplyPort->mp_SigBit);

	TimReq->tr_time.tv_secs = timeout / 10;
	TimReq->tr_time.tv_micro = (timeout % 10) * 100000;

	TimReq->tr_node.io_Command = TR_ADDREQUEST;
	TimeSig = 1 << (TimReq->tr_node.io_Message.mn_ReplyPort->mp_SigBit);

	SetSignal (0, SerSig | TimeSig);

	SendIO ((struct IORequest *)TimReq);
	SendIO ((struct IORequest *)SerReq);
	SigMask = Wait (SIGBREAKF_CTRL_C | SerSig | TimeSig);

	if ((SigMask & TimeSig) == 0)
	{
	    AbortIO ((struct IORequest *)TimReq);
	}

	if ((SigMask & SerSig) == 0)
	{
	    AbortIO ((struct IORequest *)SerReq);
	}
        WaitIO ((struct IORequest *)TimReq);
        WaitIO ((struct IORequest *)SerReq);
	if(SigMask & SIGBREAKF_CTRL_C)
	{
            return -1;
	}
    }
    else 
    {
	if (SerReq->IOSer.io_Actual > 0 || timeout == -1)
	{
	    if(timeout != -1)
	    {
                if (nchar > SerReq->IOSer.io_Actual)
	        {
		    nchar = SerReq->IOSer.io_Actual;
	        }
	    }
	    SerReq->IOSer.io_Length = nchar;
	    SerReq->IOSer.io_Data = buf;
	    SerReq->IOSer.io_Command = CMD_READ;
            SerSig = 1 << (SerReq->IOSer.io_Message.mn_ReplyPort->mp_SigBit);

	    SetSignal (0, SerSig);
            SendIO ((struct IORequest *) SerReq);
            SigMask = Wait (SIGBREAKF_CTRL_C | SerSig);

            if ((SigMask & SerSig) == 0)
	    {
	        AbortIO ((struct IORequest *) SerReq);
	    }
            WaitIO ((struct IORequest *) SerReq);
            if(SigMask & SIGBREAKF_CTRL_C)
	    {
                return -1;
	    }
	}
    }
    return SerReq->IOSer.io_Actual;
}

int SWrite (char *buf, int len)
{
    int SigMask, SerSig;

    if(!SerReq)
    {
        return -1;
    }

    if(len <= 0)
    {
        len = strlen(buf);
    }
    SerReq->IOSer.io_Length = len;
    SerReq->IOSer.io_Data = buf;
    SerReq->IOSer.io_Command = CMD_WRITE;
    SerSig = 1 << (SerReq->IOSer.io_Message.mn_ReplyPort->mp_SigBit);

    SetSignal (0, SerSig);
    SendIO ((struct IORequest *) SerReq);

    SigMask = Wait (SIGBREAKF_CTRL_C | SerSig);

    if ((SigMask & SerSig) == 0)
    {
        AbortIO ((struct IORequest *) SerReq);
    }
    WaitIO ((struct IORequest *) SerReq);
    if(SigMask & SIGBREAKF_CTRL_C)
    {
        return -1;
    }
    return 0;
}
#if 0
AWait (int t)
{
    struct timerequest *TimReq = NULL;
    int TimeSig = 0;
    int SigMask;

    if (t)
    {
	TimReq = Open_Device (TIMERNAME, sizeof (struct timerequest), UNIT_MICROHZ, 0);

	TimReq->tr_time.tv_secs = t / 1000000;
	TimReq->tr_time.tv_micro = t % 1000000;

	TimReq->tr_node.io_Command = TR_ADDREQUEST;
	TimeSig = 1 << (TimReq->tr_node.io_Message.mn_ReplyPort->mp_SigBit);

	SetSignal (0, TimeSig);
	SendIO (TimReq);
	SigMask = Wait (TimeSig);
	WaitIO (TimReq);
	Close_Device (TimReq);
    }
}
#endif
static struct IORequest *OpenIORequest (char *Name, int Size)
{
    struct MsgPort *Port;
    struct IORequest *IOReq = NULL;

    if ((Port = (struct MsgPort *) CreatePort (Name, 0)))
    {
	if ((IOReq = (struct IORequest *) CreateExtIO (Port, Size)) == NULL)
	{
	    DeletePort (Port);
	}
    }
    return IOReq;
}
static struct IORequest *DeviceOpen(char *Name, int Unit, struct IORequest * IOReq, int Flags)
{
    if (OpenDevice (Name, Unit, IOReq, Flags))
    {
	Close_Device (IOReq);
	IOReq = NULL;
    }
    return IOReq;
}
static struct IORequest *Open_Device(char *Name, int Size, int Unit, int Flags)
{
    struct IORequest * IOReq;

    IOReq = OpenIORequest(Name, Size);
    if(IOReq)
    { 
        IOReq = DeviceOpen (Name, Unit, IOReq, Flags);
    }
    return IOReq;
}
static void Close_Device (struct IORequest * IOReq)
{
    if (IOReq)
    {
	if (IOReq->io_Message.mn_ReplyPort)
	{
	    DeletePort (IOReq->io_Message.mn_ReplyPort);
	}

	if (IOReq->io_Device)
	{
	    CloseDevice (IOReq);
	}
	DeleteExtIO (IOReq);
    }
}

#ifdef DEBUG
    
main()
{
    struct IOExtSer *ps;
    static char buf[80]= {0};
    int i,n;
   
    ps = OpenSerial(0);
    printf("Open gives %x\n", ps);
    if(ps)
    {
#if 0
        SWrite("ATZ\r", 4);
        n = SRead(buf, 2, -1);
        printf("Read -1 gives %d\n", n);
        n = SRead(buf+n, 10, 0);
        printf("Read 0 gives %d\n", n);
        if(n)
        {
            puts(buf);
            for(i = 0; i < n; i++)
            {
                printf("0\%02x ", buf[i]);
            }
            putchar('\n');
        }
        n = SRead(buf, 80, 20);
        printf("Read 2 = %d\n", n);
#else
        ps->IOSer.io_Command = SDCMD_SETPARAMS;
        ps->io_SerFlags &= ~(SERF_XDISABLED | SERF_PARTY_ON);
        ps->io_SerFlags |= SERF_RAD_BOOGIE;
        DoIO(ps);
        ps->IOSer.io_Command = SDCMD_QUERY;
        DoIO(ps);
        
        printf("Baud %d\n", ps->io_Baud);
        printf("RBuf %d\n", ps->io_RBufLen);
        printf("Flags are %0x\n", ps->io_SerFlags);
        for(i = 0; i < 8; i++)
        {
            if(ps->io_SerFlags & (1 << i))
            {
                printf("     Bit %d set\n", i);
            }
        }
#endif
        n = SRead(buf, 80, 20);
        printf("n is %d\n", n);
        printf("Serial Error %d\n", ps->IOSer.io_Error);
        CloseSerial();
    }

}
#endif

