#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <ctype.h>
#include <errno.h>
#include <signal.h>
#include <stdarg.h>
#include <time.h>

#include <qdos.h>
#include <pd/csrvthg.h>

#include "qfax.h"
#include "qfaxparse.h"
#include "qfaxsrv.h"

int ServerInit(void)
{
    return UseSrvThg (SRVR, "QFAX");
}

void ServerFree(void)
{
    FreeSrvThg (SRVR, "QFAX");
}

extern char *ExitParams;

void ParseExitP(int ctx, char *srvmsg)
{
    if(ExitParams)
    {
        free(ExitParams);
    }
    if(*(srvmsg+9) == ' ')
    {
        ExitParams = strdup(srvmsg+10);
    }
    SrvRply(ctx, "OK", -1);
    free(srvmsg);
}

enum msg_states ActOnPipeMsg(int ctx, char **srvmsg, short srvlen)
{
    static char *cmds[] =
    {   "send",
        "grab",
        "suspend",
        "resume",
        "quit",
        "exit",
        "answer",
        "poll",
        "exitparam",
        NULL};
    char **pt;
    enum msg_states i;
    short dozing = 0;

    do
    {    
        for(i = P_SEND, pt = cmds; *pt; pt++, i++)
        {
            if(strncasecmp(*pt, *srvmsg, strlen(*pt)) == 0)
            {
                break;
            }
        }

        msg(0, "Server Message \"%s\"", (*pt) ? *pt : *srvmsg);
        
        if(i == P_BAD || (!dozing && i == P_WAKE))
        {
            SrvRply(ctx, "\007Bad message", -1);
            free(*srvmsg);
            *srvmsg = NULL;
        }
        else if(i == P_EPARM)
        {
            ParseExitP(ctx, *srvmsg);
        }
        else if(dozing)
        {
            if(i == P_WAKE || i == P_BYE || i == P_QUIT)
            {
                dozing = 0;
                SrvRply(ctx, "OK", 2);
                free(*srvmsg);
                *srvmsg = NULL;
                ReopenSerial();
            }
            else
            {
                SrvRply(ctx, "\007Wake required", -1);
                free(*srvmsg);
                *srvmsg = NULL;
            }
	}        
        else
        {        
            if(i == P_SLEEP)
            {
                dozing = 1;
                CloseDownSerial();
                SrvRply(ctx, "OK", 2);
                free(*srvmsg);
                *srvmsg = NULL;
            }
	}
        if(dozing)
        {
            ctx = SrvPoll("QFAX", -1, srvmsg, &srvlen);
	}
    } while (dozing);
    return i;
}
