/*--------------------.
| TTy driver for qfax |
`--------------------*/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <termios.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <sys/un.h>
#ifdef __linux__
#include <sys/ioctl.h>
#endif

struct {
	int baud;
	int code;
} speeds[] = {
	{300,	B300},
	{600,	B600},
	{1200,	B1200},
	{2400,	B2400},
	{4800,	B4800},
	{9600,	B9600},
	{19200,	B19200},
	{38400,	B38400},
	{57600,  B57600},
	{115200, B115200},
	{    0,	0}
	 
};

static int myfd = 0;
static struct termios oldtty, tty;

static ulong getspeed(int baud, ulong code)
{
    ulong n = -1;

    for (n=0; speeds[n].baud; ++n)
    {
	if(baud)
	{
	    if (speeds[n].baud == baud)
		return speeds[n].code;
	}
	else if(code)
	{
	    if (speeds[n].code == code)
		return speeds[n].baud;
	}
    }
    return -1;
}

void mode(int f, short n)
{
    static short did0 = 0;
    if(n)
    {
	if(!did0)
	{
	    (void) tcgetattr(f, &oldtty);
	    tty = oldtty;
	    tty.c_iflag = 0;
	    tty.c_lflag = 0;
	    tty.c_oflag = 0;
	    tty.c_cflag &= ~(CSIZE|PARENB);	/* disable parity */
	    tty.c_cflag |= CS8;	/* Set character size = 8 */
	    tty.c_cc[VMIN] = 1;         /* This many chars satisfies reads */
	    tty.c_cc[VTIME] = 0;	/* or in this many tenths of seconds */
	    tty.c_cflag |= CRTSCTS|CLOCAL;
	    (void) tcsetattr(f, TCSADRAIN, &tty);
	    did0 = 1;
	}
    }
    else if(did0)
    {
	(void) tcsetattr(f, TCSANOW, &oldtty);	/* Restore */
    }
}

void tty_baud(int baud)
{
    ulong bspeed;
    if((bspeed = getspeed(baud, 0)))
    {
	cfsetospeed (&tty, bspeed); 
	cfsetispeed (&tty, bspeed); 
    }
    (void) tcsetattr(myfd, TCSADRAIN, &tty);
}

/* Initialize tty device for serial file xfer */

static fd_set fds;
#ifdef USE_SOCKETS
void ManageSock(int f)
{
    FD_SET (f, &fds) ;
}
#endif

long tty_open (char *dev, long baud, short flag)
{
    static short first = 1;
    int f;

    if(first)
    {
	FD_ZERO ( &fds ) ;
	first = 0;
    }
    
    if(dev)
    {
        if((f = open(dev, O_RDWR|O_NDELAY)) > 0)
        {
            long flags;
        
            myfd = f;
            flags = fcntl( myfd, F_GETFL, 0 );
            fcntl( myfd, F_SETFL, ( flags & ~O_NDELAY ));
            mode(f, 1);
            tty_baud(baud);
        }
    }
    else
    {
	myfd = f = 0;
    }   
    FD_SET (f, &fds ) ;
#ifdef USE_SOCKETS
    if(CSOX)
    {
	FD_SET (CSOX, &fds ) ;
    }
#endif
    return f;
}

void tty_close(void)
{
    long mstat;
    
    FD_CLR(myfd, &fds);
    mode(myfd, 0);
    ioctl(myfd, TIOCMGET, &mstat);
    mstat &= ~(TIOCM_DTR|TIOCM_RTS);
    ioctl(myfd, TIOCMSET, &mstat);
    close(myfd);
    myfd = -1;
}

int SWrite(char *buf, int n)
{
    int nw,flags;
    
    if(n < 0)
    {
	n = strlen(buf);
    }

    flags = fcntl( myfd, F_GETFL, 0 );
    nw = write(myfd, buf, n);
/**********
    printf("write res %d %d %s\n", myfd, nw, 
	   (nw > 0) ? "" : strerror(errno) );
	   *****************/
    return nw;
}
#ifdef USE_SOCKETS
void HandleExtnSock()
{
    struct sockaddr_un clientname;
    int status;
    size_t size;

    size = sizeof (clientname);
    if ((status = accept (sock,
			  (struct sockaddr *) &clientname, &size)) < 0)
    {
	exit (EXIT_FAILURE);
    }
    FD_SET(status, &fds);
}
#endif
static char rbuf[1024];
static short ibuf = 0;
static char *tp = rbuf;

int SRead(char *buf, int req, int t)
{
    int n, nc,nr ;
    struct timeval t1;
    static fd_set rfds;
    short done;

    nr = 0;
    nc = req;

    t1.tv_sec  = t / 10 ; 
    t1.tv_usec = ( t % 10 ) * 100000 ;
    while(ibuf && nc)
    {
	*(buf+nr) = *tp++;
	ibuf--;
	nc--;
	nr++;
    }

    done = (nc == 0);

    while (!done)
    {
	rfds = fds;	
	n = select ( FD_SETSIZE, &rfds, 0, 0, &t1);
  	switch(n)
	{
	  case 0:
	    done = 1;
	    break;
	  case -1:
	    nr = -1;
	    done = 1;
	    break;
	  default:
	    if(FD_ISSET(myfd, &rfds))
	    {
		int nb;
		int nn;
		
		if(ioctl(myfd, FIONREAD, &nn) == 0)
		{
		    int nl = 0;
 
		    if(nn > nc)
		    {
			nl = nn - nc;
			nn = nc;
		    }			
		    if((nb = read(myfd, buf+nr, nn)) > 0)
		    {
			nr += nb;
			nc -= nb;
		    }

		    if(nr == req)
		    {
			done = 1;
			if(nl)
			{
			    if(nl > sizeof(rbuf)) nl = sizeof(rbuf);
			    ibuf = read(myfd, (tp = rbuf), nl);
			}
		    }
		}
	    }
#ifdef USE_SOCKETS
	    else if(FD_ISSET(sock, &rfds))
	    {
		HandleExtnSock();
	    }
	    else if(FD_ISSET(CSOX, &rfds)) 
	    {
		nr = ERR_VMENU; 
		done = 1;
	    }
	    else
	    {
		nr = ERR_VEXTN;
		done = 1;
	    }
#endif
	}
   }
   return nr;
}

#ifdef TEST
static int faxgets (char *s, int n, int t)
{
    int c = 0;
    char *p;
    char termc = '\n';
    short tlm, tc;
    short first = 1;
    int q = 0;

    p = s;
    *p = 0;
    tlm = t * 10;
    tc = 0;
    n--;

    while (n)
    {
	c = SRead (p, 1, 1);
	if (c < 0)
	{
	    q = -1;
	    break;
	}
	else if (c == 0)
	{
	    if (tc++ == tlm)
	    {
		q = -2;
		break;
	    }
	}
	else
	{
	    if (first)
	    {
		if (tolower (*p) == 'a')
		{
		    termc = '\r';
		}
		first = 0;
	    }
	    n--;
	    if (*p++ == termc)
	    {
		break;
	    }
	}
    }
    *p = 0;
    return q;
}

short verbose = 0;

static char *cmd (char *s, int t, char *resp)
{
    static char buf[128];
    int p;
    char *q;

    if (s && *s)
    {
	SWrite ("AT", -1);
	SWrite (s, -1);
	SWrite ("\r", -1);
	if(verbose) fprintf (stderr, "Modem : AT%s ... ", s);
    }

    while ((p = faxgets (buf, 128, t)) == 0)
    {
	if(strstr(buf, resp))
	{
	    break;
	}
    }
    return (p == 0) ? buf : "";
}


int sox[2] = {0};
int sock = 0;

int main(int ac, char **av)
{
    int f,c;
    int baud = 9600;
    char *dev = "/dev/cua0";
    int timo = 1;
    char *p;

    while((c =getopt(ac,av,"Db:d:t:v")) != EOF)
    {
	switch(c)
	{
	case 'D':
	    dev = NULL;
	    break;
	  case 'b':
	    baud = atoi(optarg);
	    break;
	  case 'd':
	    dev = optarg;
	    break;
	  case 't':
	    timo = atoi(optarg);
	    break;
	  case 'v':
	    verbose = 1;
	    break;
	}
    }
    if(verbose)
	fprintf(stderr, "testTTY %s at %d\n", dev, baud);
    
    f = tty_open(dev, baud, 1);
    SWrite("AT\r", 3);
    cmd("", timo, "OK");
    
    p = cmd ("Z", timo, "OK");
    fputs((*p) ? p : "fails\n", stderr);
    
    tty_close();
    return 0;
}
#endif
