#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>

#ifdef JLIST
#include "jlist.h"
#endif

/*
 * All for this bit of code, just because access was broken in a
 * previous release
 */
 
int exists (char *p)
{
    int chn;
    int sts = 0;

    if ((chn = open (p, O_RDONLY)) > 0)
    {
	sts = 1;
	close (chn);
    }
    return sts;
}

#ifdef JLIST

static LIST_NODE *crt_node (void *info, LIST_NODE * prev, LIST_NODE * next)
{
    LIST_NODE *new;

    if (new = (PLIST_NODE) malloc (sizeof (LIST_NODE)))
    {
	new->item = (void *) info;
	new->prev = prev;
	new->next = next;
    }
    return (new);
}

static int ins_list (void *info, LIST_HEAD * list)
{
    LIST_NODE *new;

    if (new = crt_node (info, NULL, list->head))
    {
	if (list->length)
	{
	    list->head->prev = new;
	}
	else
	{
	    list->tail = new;
	}

	list->head = new;
	list->length++;

	return (1);
    }
    else
    {
	return (0);
    }
}

static void *rem_head (LIST_HEAD * list)
{
    return (RemoveNode (list, list->head));
}

LIST_HEAD *CreateHead (void)
{
    LIST_HEAD *new;

    if (new = (LIST_HEAD *) malloc (sizeof (LIST_HEAD)))
    {
	new->length = 0;
	new->head = new->tail = (PLIST_NODE) NULL;
    }
    return (new);
}

int AppendList (void *info, LIST_HEAD * list)
{
    LIST_NODE *new;

    if (new = crt_node (info, list->tail, NULL))
    {
	if (list->length)
	{
	    list->tail->next = new;
	}
	else
	{
	    list->head = new;
	}
	list->tail = new;

	list->length++;

	return (1);
    }
    else
    {
	return (0);
    }
}

LIST_NODE *InsertBeforeNode (void *info, LIST_HEAD * list, LIST_NODE * node)
{
    LIST_NODE *tmp;
    int fnd = 0;

    for (tmp = list->head; tmp != NULL; tmp = tmp->next)
    {
	if (tmp == node)
	{
	    fnd = 1;
	    break;
	}
    }

    if (!fnd)
    {
	tmp = NULL;
    }
    else if ((tmp = crt_node (info, node->prev, node->next)))
    {
	if (node->prev != NULL)
	{
	    node->prev->next = tmp;
	}

	tmp->prev = node->prev;
	tmp->next = node;
	node->prev = tmp;

	list->length++;

	if (list->head == node)
	{
	    list->head = tmp;
	}
    }
    return (tmp);
}

void *RemoveNode (LIST_HEAD * list, LIST_NODE * node)
{
    void *info;

    if (node == NULL)
    {
	return (NULL);
    }

    info = node->item;

    if (node == list->head)
    {
	list->head = node->next;
	if (list->head)
	{
	    list->head->prev = NULL;
	}
    }
    else
    {
	node->prev->next = node->next;
    }

    if (node == list->tail)
    {
	list->tail = node->prev;

	if (list->tail)
	{
	    list->tail->next = NULL;
	}
    }
    else
    {
	node->next->prev = node->prev;
    }
    list->length--;

    free (node);

    return (info);
}

LIST_NODE *FindNode (LIST_HEAD * list, int (*cmpfnc) (void *, void *), void *info)
{
    LIST_NODE *tmp;

    for (tmp = list->head; tmp != NULL; tmp = tmp->next)
    {
	if ((*cmpfnc) (tmp->item, info) == 0)
	{
	    break;
	}
    }

    return (tmp);
}

int DeleteList (LIST_HEAD * list, short flag)
{
    void *info;

    while (list->head)
    {
	info = rem_head (list);
	if (info && flag)
	{
	    free (info);
	}
    }
    free ((char *) list);
}

#if 0
LIST_NODE *InsertAfterNode (void *info, LIST_HEAD * list, LIST_NODE * node)
{
    LIST_NODE *tmp;
    int fnd = 0;

    for (tmp = list->head; tmp != NULL; tmp = tmp->next)
    {
	if (tmp == node)
	{
	    fnd = 1;
	    break;
	}
    }

    if (!fnd)
    {
	tmp = NULL;
    }
    else if ((tmp = crt_node (info, node->prev, node->next)))
    {
	tmp->next = node->next;
	node->next = tmp;

	tmp->prev = node;

	if (node->next != NULL)
	{
	    node->next->prev = tmp;
	}

	list->length++;

	if (list->tail == node)
	{
	    list->tail = tmp;
	}
    }

    return (tmp);
}


void *rem_tail (list)
LIST_HEAD *list;
{
    return (rem_node (list, list->tail));
}

#endif

#define QFAX 1
#ifdef QFAX
#include <string.h>
#include <qfv.h>

#ifdef QDOS
#define EXT_SEP '_'
#define access(p1, p2) !exists(p1)
#else
#define EXT_SEP '.'
#endif

static char Stem[40] =
{0};
static LIST_HEAD *plh = NULL;
static LIST_NODE *ppn = NULL;
int nStem;

#define LISTMAX 16*1024

static int CheckItem (char *a, char *b)
{
    return (strcmp (a, b) < 0);
}

static char *SplitFile (char *fname, int flag)
{
    char *item = NULL;
    char *s;

    s = strrchr (fname, EXT_SEP);

    if (s && atoi (s + 1))
    {
	if (s)
	{
#ifdef QDOS
	    while (s >= fname && *(--s) != '_')
#else
	    while (s >= fname && *(--s) != '/' && *s != ':')
#endif
		;		/* loop back */
	    s++;
	}

	if (flag)
	{
	    item = strdup (s);
	}
	else
	{
	    item = s;
	}
    }
    return item;
}

static int CheckFiles (LIST_HEAD * ph, LIST_NODE * pn)
{
    int flag = 0;
    LIST_NODE *x;
    char *p;
    int n;

    if (p = strchr (pn->item, EXT_SEP))
    {
	n = p - (char *) pn->item;
    }
    else
    {
	n = strlen (pn->item);
    }
    if (x = pn->prev)
    {
	if (strncmp (pn->item, x->item, n) == 0)
	{
	    flag |= M_PPREV;
	    for (; x = x->prev;)
	    {
		if (strncmp (pn->item, x->item, n))
		{
		    break;
		}
	    }
	}
	if (x)
	{
	    flag |= M_FPREV;
	}
    }
    if (x = pn->next)
    {
	if (strncmp (pn->item, x->item, n) == 0)
	{
	    flag |= M_PNEXT;
	    for (; x = x->next;)
	    {
		if (strncmp (pn->item, x->item, n))
		{
		    break;
		}
	    }
	}
	if (x)
	{
	    flag |= M_FNEXT;
	}
    }
    return flag;
}

int BuildFileList (char *fn)
{
    char *px, *xx;
    char *item;
    int n = 0;

    if (plh)
    {
	DeleteList (plh, 1);
    }

    item = SplitFile (fn, 0);
    if (item)
    {
	nStem = item - fn;
	if (nStem)
	{
	    strncpy (Stem, fn, nStem);
	}
#ifdef QDOS
	strcpy (Stem + nStem, "*");
#else
	strcpy (Stem + nStem, "#?");
#endif

	for (n = LISTMAX; n > 1023; n = n >> 1)
	{
	    if ((px = xx = malloc (n)) != NULL)
	    {
		break;
	    }
	}
	if (px == NULL)
	{
	    return 0;
	}

#ifdef _DCC
#define QDR_DATA 0
#endif

	if (getfnl (Stem, px, n, QDR_DATA) > 1)
	{

	    plh = CreateHead ();

	    for (px = xx; *px; px += strlen (px) + 1)
	    {
		item = SplitFile (px, 1);
		if (item)
		{
                    strcpy(Stem+nStem, item);
#ifdef TEST
                    printf("Stemmng %s\n", Stem);
#endif
                    if(access(Stem, 0) == 0)
                    {
		    if (plh->length == 0)
		    {

			AppendList (item, plh);
		    }
		    else
		    {
			ppn = FindNode (plh, CheckItem, item);
			if (ppn)
			{
			    InsertBeforeNode (item, plh, ppn);
			}
			else
			{
			    AppendList (item, plh);
			}
		    }
		}
	        }
	    }
	    free (xx);

	    ppn = FindNode (plh, CheckItem, fn + nStem);
	    n = CheckFiles (plh, ppn);
	}
    }
    return M_LOAD | n;
}

#define MOVEITEM(p1, p2) ((p2 == I_FPREV) ? (p1)->prev : (p1)->next)

static void SearchList (short n, char *q, short nlen)
{
    LIST_NODE *p;

    while (ppn = MOVEITEM (ppn, n))
    {
	if (strncmp (ppn->item, q, nlen))
	{
	    break;
	}
    }
    if (n == I_FPREV)
    {
	p = ppn->prev;
	while (p && (strncmp (ppn->item, p->item, nlen) == 0))
	{
	    ppn = p;
	    p = p->prev;
	}
    }
}

void GetFromList (PVIEWPORT pv, short n)
{
    char *p, *q;
    int nlen;

    q = pv->FileName + nStem;
    p = strchr (q, '_');
    nlen = p - q;

    switch (n)
    {
	case I_PPREV:
	    ppn = ppn->prev;
	    break;
	case I_PNEXT:
	    ppn = ppn->next;
	    break;
	case I_FPREV:
	case I_FNEXT:
	    SearchList (n, q, nlen);
	    break;
    }
    strncpy (pv->FileName, Stem, nStem);
    strcpy (pv->FileName + nStem, ppn->item);
}

#endif
#ifdef TEST
main (int ac, char **av)
{

    BuildFileList(av[1]);

    for (ppn = plh->head; ppn; ppn = ppn->next)
    {
	int n;

	n = CheckFiles (plh, ppn);
	fputs (ppn->item, stdout);
	fputs ((n & M_PPREV) ? " -page " : "       ", stdout);
	fputs ((n & M_PNEXT) ? " +page " : "       ", stdout);
	fputs ((n & M_FPREV) ? " -file " : "       ", stdout);
	fputs ((n & M_FNEXT) ? " +file " : "       ", stdout);
	fputc ('\n', stdout);
    }

}
#endif
#endif
