/*******************************************************************************
*
*   Given a pointer to a menu sub-window working definition for a regular
*   spacing menu this function checks whether the menu should be scrollable.
*   It adjusts the menu window size if it adds or takes away a scroll bar.
*   It sets the x size and spacing constants.
*
*******************************************************************************/

#include <stdio.h>
#include <qdos.h>
#include <qptr.h>
#include "qfv.h"

Prototype void menu_chks (struct WM_menw *);
Prototype void menu_rems (struct WM_menw *);

void menu_chks (menu)
struct WM_menw *menu;
{
    struct WM_pwcb *pwcb;
    short spacing;

    pwcb = menu->ypwcb;		/* part window control block (scroll) */
    spacing = -menu->ys.c._spce;
/*    Report("Size %d %d %d", menu->nrow, menu->ysize, spacing); */
    
    if (menu->nrow > menu->ysize / spacing)
    {
	if (pwcb->nsec == 0)
	    menu->xsize -= WW_XBAR;	/* adjust width for bar */
	pwcb->nsec = 1;		/* one section */
	pwcb->s[0].stt = 0;	/* from zero */
	pwcb->s[0].siz = ((menu->ysize - WW_YARROW * 2) / spacing);	/* size */
	menu->yoff = WW_YARROW;	/* offset for arrows */
    }
    else
    {
	if (pwcb->nsec)
	    menu->xsize += WW_XBAR;	/* adjust width for no bar */
	pwcb->nsec = 0;		/* no sections */
	menu->yoff = 0;		/* no arrows, no offset */
    }
    menu->xs.c._spce = -(menu->xsize) / menu->ncol;	/* width of each column */
    menu->xs.c._size = menu->xs.c._spce + menu->curw * 4;	/* ... item border */
}

/*******************************************************************************
*
*   Given a pointer to a menu sub-window working definition for a scrollable
*   menu this function removes the scroll bar and arrows (if present)
*
*******************************************************************************/

void menu_rems (menu)
struct WM_menw *menu;
{
    struct WM_pwcb *pwcb;

    pwcb = menu->ypwcb;		/* part window control block (scroll) */

    if (pwcb->nsec)
    {
	menu->xsize += WW_XBAR;	/* adjust width for no bar */
	pwcb->nsec = 0;		/* no sections */
	menu->yoff = 0;
    }				/* no arrows, no offset */
}
