#include "forms.h"
#include "xfax.h"

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>

#include "xfax_p.h"
#include "qfv.h"

#include "right.xpm"
#include "left.xpm"
#include "rarrow.xpm"
#include "larrow.xpm"
#include "folder.xpm"
#include "print.xpm"
#include "info.xpm"

void DrawImage(FD_fax *f)
{
    BITMAP *b;

    b = (BITMAP*) f->vdata;

    if(b->pixmap && b->pgc && b->xi)
    {
	char buf[64];
	char *p;
	sprintf(buf, "Page: %d/%d",  1+pv->FaxPage, pv->qh.pagemax);

	XPutImage (fl_get_display(), b->pixmap, b->pgc, b->xi, 
	       0, 0, 0, 0, b->width, b->height);
	XCopyPlane (fl_get_display(), b->pixmap, CANVID(f->canvas), 
		    b->sgc, b->curx, b->cury, b->width,
		    b->height, 0, 0, 1);

	fl_set_object_label(f->pages, buf);
	if((p = strrchr(pv->FileName, '/')))
	{
	    p++;
	    fl_set_object_label(f->fnam, p);
	}

    }
}

void RefreshImage(FD_fax *f)
{
    BITMAP *b;

    b = (BITMAP*) f->vdata;
    if(b->pixmap && b->sgc && b->xi)
    {
	XCopyPlane (fl_get_display(), b->pixmap, CANVID(f->canvas), 
		    b->sgc, b->curx, b->cury, b->width,
		    b->height, 0, 0, 1);
    }
}

int canvas_expose(FL_OBJECT *ob, Window win, int w, int h, XEvent *ev, void *d)
{
    BITMAP *b;    
    FD_fax *f = d;

    XExposeEvent *xe = (XExposeEvent *)ev;
    b =  (BITMAP *) f->vdata;

    if(b->fn)
    {
	Dimension xx,yy;
	if((xx = xe->x+xe->width) > b->width)
	{
	    XFillRectangle(fl_get_display(), win, b->sgc,
		       b->width, xe->y, xx - b->width, xe->height);
	}
	if ((yy = xe->y + xe->height) > b->height)	
	{
	    XFillRectangle(fl_get_display(), win, b->sgc,
		       xe->x, b->height, xe->width, yy - b->height);
	}
	XCopyPlane (fl_get_display(), b->pixmap, win, 
		    b->sgc, b->curx+xe->x, b->cury+xe->y, 
		    xe->width, xe->height, xe->x, xe->y, 1);


    }
    else
    {
	XFillRectangle(fl_get_display(), win, b->sgc,
		       xe->x, xe->y, xe->width, xe->height);
    }
    return 0;
}

int init_canvas (FL_OBJECT *ob)
{
    int vmode = fl_get_vclass();
    FD_fax *f;
    BITMAP *b;    

    f = ob->form->u_vdata;
    b =  (BITMAP *) f->vdata;
    b->sgc = fl_state[vmode].gc[7]; 
    fl_set_foreground(b->sgc, FL_WHITE);
    fl_add_canvas_handler(f->canvas, Expose, canvas_expose, f);
    b->vres = 1;
    return 0;
}    

static int post_handler(FL_OBJECT *ob, int event,
            FL_Coord mx, FL_Coord my, int key, void *xev)
{
    char s[128];
    FD_fax *f;

    f = (FD_fax *)ob->form->u_vdata;

    *s = '\0';

    if(f)
    {
	if(ob == f->button1)
	{
	    strcpy(s, "Open a new File");
	}
	else if(ob == f->button2)
	{
	    strcpy(s, " Left: Print page \nRight: Print fax");
	    if(f->button2->u_ldata == 0) *s = '*';
	}
	else if(ob == f->button3)
	{
	    strcpy(s, " Previous Fax ");
	    if(f->button3->u_ldata == 0) *s = '*';
	}
	else if(ob == f->button4)
	{
	    strcpy(s, " Previous page ");
	    if(f->button4->u_ldata == 0) *s = '*';
	}
	else if(ob == f->button5)
	{
	    strcpy(s, " Next page ");
	    if(f->button5->u_ldata == 0) *s = '*';
	}
	else if(ob == f->button6)
	{
	    strcpy(s, " Next Fax ");
	    if(f->button6->u_ldata == 0) *s = '*';
	}
	else if(ob == f->button7)
	{
	    strcpy(s, " Fax Info ");
	    if(f->button7->u_ldata == 0) *s = '*';
	}
	else if(ob == f->vres)
	{
	    strcpy(s, "Toggle display resolution");
	}

	if(event == FL_ENTER && *s)
	{
	    fl_show_oneliner(s, ob->form->x + ob->x,
			     ob->form->y + ob->y + ob->h + 5);
	}
	else
	{
	    if(event != FL_MOTION)
		fl_hide_oneliner();
	}
    }
    return 0;
}


#define MO "Load|Print Page|Print Fax|Next Page|Prev Page|Next Fax|Prev Fax|Flip|Quit"

void init_form (FD_fax *f)
{
    short j;

    fl_set_pixmap_data (f->button1, folder_xpm);
    fl_set_pixmap_data (f->button2, print_xpm);

    fl_set_pixmap_data (f->button3, larrow_xpm);
    fl_set_pixmap_data (f->button4, left_xpm);
    fl_set_pixmap_data (f->button5, right_xpm);
    fl_set_pixmap_data (f->button6, rarrow_xpm);
    fl_set_pixmap_data (f->button7, info_xpm);

    fl_set_object_posthandler(f->button1, post_handler);
    fl_set_object_posthandler(f->button2, post_handler);
    fl_set_object_posthandler(f->button3, post_handler);
    fl_set_object_posthandler(f->button4, post_handler);
    fl_set_object_posthandler(f->button5, post_handler);
    fl_set_object_posthandler(f->button6, post_handler);
    fl_set_object_posthandler(f->button7, post_handler);
    fl_set_object_posthandler(f->vres, post_handler);

    fl_set_menu(f->menu, MO);

    for(j = 2; j < 9; j++)
    { 
	fl_set_menu_item_mode (f->menu, j, FL_PUP_GREY);
    }
}


int main(int argc, char *argv[])
{
   FD_fax *fd_fax;
   Dimension w,h;

   fl_initialize(&argc, argv, "XQfax", 0, 0);
   fd_fax = create_form_fax();
   fd_fax->fax->u_vdata = fd_fax;

   h = DisplayHeight (fl_get_display(), fl_screen);
   w = DisplayWidth (fl_get_display(), fl_screen);

   init_xfax(fd_fax, argc, argv);

   fl_set_form_size (fd_fax->fax, (w*90)/100, (h*90)/100);
   fl_set_slider_size(fd_fax->xs, 1.0);
   fl_set_slider_size(fd_fax->ys, 1.0);

   init_form(fd_fax);
   fl_show_form(fd_fax->fax,FL_PLACE_CENTERFREE, FL_FULLBORDER, "xqfax");
   init_canvas(fd_fax->canvas);

   GetBitMap (fd_fax, fd_fax->vdata, pv->FileName);

   fl_do_forms();
   return 0;
}

